//=======================================================================================
// ____          ____    __    ______     __________   __      __       __        __
// \    \       |    |  |  |  |   _   \  |___    ___| |  |    |  |     /  \      |  |
//  \    \      |    |  |  |  |  |_)   |     |  |     |  |    |  |    /    \     |  |
//   \    \     |    |  |  |  |   _   /      |  |     |  |    |  |   /  /\  \    |  |
//    \    \    |    |  |  |  |  | \  \      |  |     |   \__/   |  /  ____  \   |  |____
//     \    \   |    |  |__|  |__|  \__\     |__|      \________/  /__/    \__\  |_______|
//      \    \  |    |   ________________________________________________________________
//       \    \ |    |  |  ______________________________________________________________|
//        \    \|    |  |  |         __          __     __     __     ______      _______
//         \         |  |  |_____   |  |        |  |   |  |   |  |   |   _  \    /  _____)
//          \        |  |   _____|  |  |        |  |   |  |   |  |   |  | \  \   \_______
//           \       |  |  |        |  |_____   |   \_/   |   |  |   |  |_/  /    _____  |
//            \ _____|  |__|        |________|   \_______/    |__|   |______/    (_______/
//
//  This file is part of VirtualFluids. VirtualFluids is free software: you can
//  redistribute it and/or modify it under the terms of the GNU General Public
//  License as published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
//  VirtualFluids is distributed in the hope that it will be useful, but WITHOUT
//  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//  for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with VirtualFluids (see COPYING.txt). If not, see <http://www.gnu.org/licenses/>.
//
//! \author Martin Schoenherr
//=======================================================================================
#ifndef FILE_PART_CALCULATOR
#define FILE_PART_CALCULATOR

#include <basics/DataTypes.h>

//! \brief calculations needed for writing the simulation's output into multiple file parts
//! \details To prevent output files from becoming to large, they are split into multiple part files. For this process some
//! calculations are needed.
class FilePartCalculator
{
public:
    //! \brief calculate how many output vtk-files are created for one timestep of the given grid level
    static uint calculateNumberOfParts(uint numberOfNodes);
    //! \brief calculate how many grid nodes are written to the file with the given index
    static uint calculateNumberOfNodesInPart(uint numberOfNodes, uint indexOfFilePart);
    //! \returns index of the first node in this file part
    static uint calculateStartingPostionOfPart(uint indexOfPart);

    //! \brief limits how many grid nodes are written into a single vtk file
    static const uint limitOfNodesForVTK = 30e6; // max 30 million nodes per VTK file
};

#endif
